
; *
; * GB Printer Test
; *  written by ClapOn/ClapOff, date980815
; *
; *  Special thanks to Martin Eyre for figuring
; *   out alot of the GB Printer format.
; *
; * Compiled with RGBAsm v1.08
; *

        INCLUDE "hardware.inc"
        INCLUDE "load1.inc"


        SECTION "Low Ram",BSS

Test:   DB


        SECTION "Org $0",HOME
        ret

        SECTION "Org $100",HOME[$100]

;*** Beginning of rom execution point ***

        nop
        jp      Start

        NINTENDO_LOGO                   ; Nintendo graphic logo

;Rom Header Info

;    0123456789ABCDEF
 DB "GB PRINTER TEST",0     ; Cart name   16bytes
 DB $36,$30,3                 ; Not used
 DB CART_ROM_MBC1                  ; Cart type
 DB CART_ROM_256K             ; ROM Size (in bits)
 DB CART_RAM_NONE             ; RAM Size (in bits)
 DB 1,$33                     ; Maker ID
 DB 0                         ; Version     =0
 DB $e2                       ; Complement check (important)
 DW $c40e                     ; Checksum (not important)

        INCLUDE "memory1.asm"   ; memory set/copy

Start:
        di
        ld      sp,$ffff

        call    DisplayPicture        ; Display picture on screen
        call    PrintScreenAndWait    ; Print what's on screen
LoopForever:
        jr      LoopForever



; * Display a picture *

DisplayPicture:
        call    DisplayOff

        ld      hl,PicTileSet
        ld      de,$8000
        ld      bc,$1000
        call    mem_Copy        ; Copy tile set to memory

        ld      hl,PicMap
        ld      de,$9800
        call    Copy20x18       ; Copy tile map to memory

        ld      a,$e4
        ldh     [rBGP],a        ; Setup default background colors

        xor     a
        ldh     [rSCX],a
        ldh     [rSCY],a

        ld      a,LCDCF_ON|LCDCF_BG8000|LCDCF_BG9800|LCDCF_BGON|LCDCF_OBJ16|LCDCF_OBJOFF
        ldh     [rLCDC],a         ; Turn screen on

        ret

; Copy char map to screen
; Entry: HL = Source Map
;        DE = Destination Screen

Copy20x18:
        ld      b,18
Copy20xB:
        ld      c,20
Copy202:
        ld      a,[hl+]
        ld      [de],a
        inc     de
        dec     c
        jr      nz,Copy202

        push    hl
        ld      hl,12
        add     hl,de
        ld16    de,hl
        pop     hl

        dec     b
        jr      nz,Copy20xB

        ret

; *** Turn off LCD Display ***

DisplayOff:
        ldh     a,[rLCDC]
        rlca                    ; Is display already off?
        ret     nc              ; yes, exit

WaitForVBLK:
        ldh     a,[rLY]
        cp      $91
        jr      nz,WaitForVBLK

        ldh     a,[rLCDC]
        and     $7f
        ldh     [rLCDC],a

        ret

; ** Check to see if printer is done **
; Exit: Z flag set if done.

PrinterCheck:
        ld      hl,prnt_Status
        call    SendPrtrCommand

        ld      a,e
        and     $2
        ret

; *** Print the picture on the screen ***
; *** at $9800 using tile set at $8000 ***
;
; Exit as soon as printing ends.
;
; Exit: A = 0 - No errors
;       A = 1 - Batteries Low
;       A = 2 - Check Link Cable
;       A = 3 - Paper Jam
;       A = 4 - Too Hot or Cold to Operate

PrintScreenAndWait:
        call    PrintScreen             ; Start screen printing
        or      a                       ; Any errors?
        ret     nz                      ; yes, exit

PSW2:   call    PrinterCheck
        ld      b,a

        ld      a,c
        or      a                       ; Did printer ack the status?
        jr      nz,LinkCableError       ; no

        ld      a,d
        and     $f0
        cp      $80                     ; Did printer ack the status?
        jr      nz,LinkCableError       ; no

        call    CheckForErrors

        ld      a,b
        or      a                       ; Is printer done?
        jr      nz,PSW2                 ; not yet

        ret                             ; A = 0 means no errors

; *** Print the picture on the screen ***
; *** at $9800 using tile set at $8000 ***
;
; Exit as soon as printing starts.
;
; Exit: A = 0 - No errors
;       A = 1 - Batteries Low
;       A = 2 - Check Link Cable
;       A = 3 - Paper Jam
;       A = 4 - Too Hot or Cold to Operate

PrintScreen:

        ld      hl,prnt_Init
        call    SendPrtrCommand         ; Send printer init string

        ld      a,c
        or      a                       ; Did printer ack the status?
        jr      nz,LinkCableError       ; no

        ld      a,d
        and     $f0
        cp      $80                     ; Did printer acknowledge data?
        jr      nz,LinkCableError       ; no

        call    CheckForErrors

        ld      hl,$9800
        ld      b,18/2
PrntScrn2:
        push    bc
        push    hl
        call    PrintTwoLines           ; Send two rows of data to printer

        ld      hl,prnt_Status          ; Get printer status info.
        call    SendPrtrCommand
        call    CheckForErrors

        pop     hl
        pop     bc

        ld      de,32*2
        add     hl,de

        dec     b                       ; Have all 18 lines been sent?
        jr      nz,PrntScrn2            ; not yet


        ld      hl,prnt_EndOfData       ; Not sure what this is for
        call    SendPrtrCommand

        ld      hl,prnt_Start           ; Tell printer to start printing
        call    SendPrtrCommand

PrntScrn3:
        ld      hl,prnt_Status
        call    SendPrtrCommand
        call    CheckForErrors

        ld      a,e
        and     $2                      ; Has printing begun?
        jr      z,PrntScrn3             ; not yet

        xor     a                       ; A=0 means no errors
        ret

; If print error found, pop calling address & return

CheckForErrors:
        bit     7,e
        jr      nz,BattLowError
        bit     6,e
        jr      nz,TemperatureError
        bit     5,e
        jr      nz,PaperJamError
        ret

BattLowError:
        pop     af
        ld      a,1
        ret

LinkCableError:
        ld      a,2
        ret

PaperJamError:
        pop     af
        ld      a,3

TemperatureError:
        pop     af
        ld      a,4
        ret

; *** Send a command string to printer ***
; Entry: HL = Ptr to command string
; Exit:  C = Byte two places before last byte
;        D = Byte one place before last byte
;        E = Last byte from printer

SendPrtrCommand:
        ld      b,[hl]
        inc     hl
SendTP2:
        ld      a,[hl+]
        ldh     [rSB],a

        ld      a,$81
        ldh     [rSC],a         ; Start xfer, internal clk

SendTP3:
        ldh     a,[rSC]
        and     $80             ; Is xfer done?
        jr      nz,SendTP3      ; not yet

        ld      c,d
        ld      d,e

        ldh     a,[rSB]
        ld      e,a

        dec     b               ; Have all bytes been sent in this command?
        jr      nz,SendTP2      ; not yet

        ret

; *** Send two rows of data to printer ***
; Entry: HL = Ptr to data

PrintTwoLines:
        push    hl
        ld      hl,prnt_2RowsData
        call    SendPrtrCommand
        pop     hl

        ld      de,0            ; Set CRC = 0

        ld      b,2

PTL2:   ld      c,20

PTL3:   lcd_WaitVRAM
        ld      a,[hl+]
        push    hl

        ld      l,a
        ld      h,0
        add     hl,hl           ; hl = A * 2
        add     hl,hl           ; hl = A * 4
        add     hl,hl           ; hl = A * 8
        add     hl,hl           ; hl = A * 16
        push    de
        ld      de,$8000
        add     hl,de           ; hl = addr of tile to print
        pop     de

        push    bc
        ld      b,16
PTL4:
        lcd_WaitVRAM
        ld      a,[hl+]
        ld      c,a
        add     e
        ld      e,a
        jr      nc,PTL5
        inc     d
PTL5:                          ; de = de + [hl]

        ld      a,c
        call    SendByte        ; Send A to printer

        dec     b               ; Are we done sending this tile?
        jr      nz,PTL4         ; not yet

        pop     bc

        pop     hl
        dec     c               ; Are we done printing this row?
        jr      nz,PTL3         ; not yet

        push    de
        ld      de,12
        add     hl,de
        pop     de              ; hl = ptr to next row

        dec     b               ; Are we done with both rows?
        jr      nz,PTL2         ; not yet

        ld      hl,$04+$00+$80+02       ; Include prnt_2RowsData command in CRC
        add     hl,de

        ld      a,l
        call    SendByte        ; Send CRC low byte
        ld      a,h
        call    SendByte        ; Send CRC high byte

        xor     a
        call    SendByte        ; Send 0x00
        xor     a
        call    SendByte        ; Send 0x00

        ret


; Send A to printer

SendByte:
        ldh     [rSB],a

        ld      a,$81
        ldh     [rSC],a         ; Start xfer, internal clk

SendByt2:
        ldh     a,[rSC]
        and     $80             ; Is xfer done?
        jr      nz,SendByt2     ; not yet

        ret


; *** Printer command strings ***

;
; 2RowsData -
;
;   DB $88,$33
;   DB $04     - Data Packet command
;   DB $00
;   DW 640     - Data packet length ($0280)
;   DS 640     - 640 bytes of data (16 byte tile * 20 columns * 2 rows)
;   DW CRC     - CRC = $04+$00+$80+$02+value of each data byte
;   DW 0
;
; STATUS -
;
;   The last two bytes of all commands returns some form
;  of status bytes. The Status Command basically requests these
;  same bytes but causes no changes by sending the command.
;
;  Bytes that are returned during transferring a command to
; the GB Printer are all 0x00 except for the last two bytes.
; The next to last byte is either 0x80 or 0x81 and the
; last byte has the following status format:
;
;  7 = Error #1 - Low Batteries (when set)
;  6 = Error #4 - Too Hot or Cold to Operate (when set)
;  5 = Error #3 - Paper Jam (when set)
;  4 = ?
;  3 = ?
;  2 = ?
;  1 = Printing In Progress (when set)
;  0 = ?
;
;  Any command packet which contains a CRC value is discarded
; if the information in the packet does not add up to the
; same value as the CRC.

prnt_Init:
        DB      10      ;Length

        DB      $88,$33
        DB      $01     ; Print Init Command
        DB      $00
        DB      $00
        DB      $00
        DW      1       ; CRC = 1+0+0+0
        DB      00,$00

prnt_2RowsData:
        DB      6       ;Length

        DB      $88,$33
        DB      $04     ; Data Packet Command
        DB      $00
        DW      640     ; Data length

prnt_EndOfData:
        DB      10      ;Length

        DB      $88,$33
        DB      $04     ; Data Packet Command
        DB      $00
        DB      $00
        DB      $00
        DW      $04     ; CRC = 4+0+0+0
        DB      $00,$00

prnt_Status:
        DB      10      ;Length

        DB      $88,$33
        DB      $0f     ; Status Check Command
        DB      $00
        DB      $00
        DB      $00
        DW      $0f     ; CRC = $f+0+0+0
        DB      $00,$00

prnt_Start:
        DB      14      ;Length

        DB      $88,$33
        DB      $02     ; Start Print Command
        DB      $00
        DB      $04     ;
        DB      $00     ;
        DB      $01     ;
        DB      $13     ; Linefeeds before (high nibble) & after (low nibble) print
        DB      $e4     ; Color Palette to use for Data (same format as GB palettes)
        DB      $40     ; Exposure ( <$40 = lighter, >$40 = darker, bit 7 not used)
        DW      $13e    ; 13e CRC = $02+$04+$01+$13+$e4+$40
        DW      0


PicTileSet:
        INCBIN  "gbp.til"

PicMap:
        INCBIN  "gbp.map"



